package com.xxfc.platform.activity.biz;


import com.github.wxiaoqi.security.admin.dto.UserMemberDTO;
import com.github.wxiaoqi.security.admin.feign.UserFeign;
import com.github.wxiaoqi.security.common.biz.BaseBiz;
import com.github.wxiaoqi.security.common.exception.BaseException;
import com.github.wxiaoqi.security.common.msg.ObjectRestResponse;
import com.github.wxiaoqi.security.common.util.process.ResultCode;
import com.xxfc.platform.activity.entity.ActivityInfo;
import com.xxfc.platform.activity.entity.ActivityInfoUser;
import com.xxfc.platform.activity.mapper.ActivityInfoUserMapper;
import com.xxfc.platform.activity.vo.ActivityInfoRuleVo;
import com.xxfc.platform.order.entity.BaseOrder;
import com.xxfc.platform.order.entity.OrderActivityDetail;
import com.xxfc.platform.order.feign.OrderFeign;
import com.xxfc.platform.order.pojo.order.ActivityInfoBO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户活动表
 *
 * @author zjw
 * @email 18178966185@163.com
 * @date 2019-07-16 14:04:22
 */
@Service
@Slf4j
public class ActivityInfoUserBiz extends BaseBiz<ActivityInfoUserMapper, ActivityInfoUser> {

    @Autowired
    ActivityInfoBiz activityInfoBiz;

    @Autowired
    OrderFeign orderFeign;

    @Autowired
    UserFeign userFeign;




    //参与活动
    @Transactional(rollbackFor = Exception.class)
    public  BaseOrder joinActivity(ActivityInfoUser activityInfoUser){
        ActivityInfo activityInfo=activityInfoBiz.selectById(activityInfoUser.getActivityId());
        if (activityInfo == null || activityInfo.getStatus() != 1)
            throw  new BaseException("无效的活动",ResultCode.NULL_CODE);
        Integer joinNumber = activityInfo.getJoinNumber() == null ? 0 : activityInfo.getJoinNumber();
        Integer number = activityInfo.getNumber() == null ? 0 : activityInfo.getNumber();
        if(number > 0 && joinNumber >= number)
            throw  new BaseException("----活动已超过参与人数----number==="+number,ResultCode.FAILED_CODE);
        Integer activityId=activityInfo.getId();
        activityInfoUser.setActivityId(activityId);
        activityInfoUser.setStatus(1);
        Long num = selectCount(activityInfoUser);
        if(num != null && num > 0)
            throw  new BaseException("用户已参与该活动",ResultCode.FAILED_CODE);
        activityInfoUser.setStatus(0);
        ActivityInfoUser activityInfoUserStatus  = getOne(activityInfoUser);
        if (activityInfoUserStatus != null ){
            activityInfoUserStatus.setStatus(2);
            updateSelectiveById(activityInfoUserStatus);
        }else {
            joinNumber += 1 ;
        }
        BaseOrder baseOrder = addOrder(activityInfo);
        if (baseOrder == null)
            throw  new BaseException("下单失败",ResultCode.FAILED_CODE);
        activityInfoUser.setOrderId(baseOrder.getId());
        activityInfoUser.setOrderNo(baseOrder.getNo());
        insertSelective(activityInfoUser);
        activityInfo.setJoinNumber(joinNumber);
        activityInfoBiz.updateSelectiveById(activityInfo);
        return baseOrder;
    }


    public  ActivityInfoUser getOne(ActivityInfoUser activityInfoUser){
        Example example = new Example(ActivityInfoUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("activityId",activityInfoUser.getActivityId()).andEqualTo("userId",activityInfoUser.getUserId()).andEqualTo("isDel",0);
        if (activityInfoUser.getStatus() != null )
            criteria.andEqualTo("status",activityInfoUser.getStatus());
        example.setOrderByClause("id desc");
        List<ActivityInfoUser> list=mapper.selectByExample(example);
        if(list.size()>0){
            return  list.get(0);
        }
        return  null;

    }



    public BaseOrder addOrder(ActivityInfo activityInfo){
        ActivityInfoBO activityInfoBO = new ActivityInfoBO();
        activityInfoBO.setActivity(activityInfo);
        ObjectRestResponse<BaseOrder> restResponse = orderFeign.add(activityInfoBO);
        if (restResponse.getData() == null ){
            throw  new BaseException(restResponse.getMessage(),restResponse.getStatus());
        }
        return  restResponse.getData();
    }


    public  ActivityInfo getInfo(ActivityInfoUser activityInfoUser){
        ActivityInfo activityInfo=activityInfoBiz.getActivityH5(activityInfoUser.getType());
        activityInfoUser.setActivityId(activityInfo.getId());
        activityInfoUser.setStatus(1);
        activityInfoUser = getOne(activityInfoUser);
        Integer isJoin = 0 ;
        if(activityInfoUser != null ){
            isJoin = 1;
        }
        activityInfo.setIsJoin(isJoin);
        return  activityInfo;
    }


    public  ObjectRestResponse   updActivty(ActivityInfo activityInfo){
        Integer orderId = activityInfo.getOrderId();
        ActivityInfoUser activityInfoUser = new ActivityInfoUser();
        activityInfoUser.setOrderId(orderId);
        activityInfoUser = selectOne(activityInfoUser);
        if (activityInfoUser == null ){
            throw  new BaseException("未参加活动",ResultCode.FAILED_CODE);
        }
        activityInfoUser.setStatus(1);
        updateSelectiveById(activityInfoUser);
        Integer userId = activityInfoUser.getUserId() == null ? 0 : activityInfoUser.getUserId();
        ActivityInfoRuleVo activityInfoRuleVo = activityInfo.getActivityInfoRuleVo();
        Integer days = 0;
        if (activityInfoRuleVo != null ){
            days = activityInfoRuleVo.getDays();
        }
        UserMemberDTO userMemberDTO = new UserMemberDTO() {{
            setUserId(userId);
            setIsBind(ISBIND_BIND);
        }};
        userMemberDTO.setRentFreeDays(days);
        userMemberDTO.setTotalNumber(days);
        return userFeign.buyMember(userMemberDTO);
    }



}