package com.xxfc.platform.order.rest;

import com.github.wxiaoqi.security.admin.feign.UserFeign;
import com.github.wxiaoqi.security.auth.client.annotation.IgnoreClientToken;
import com.github.wxiaoqi.security.auth.client.annotation.IgnoreUserToken;
import com.github.wxiaoqi.security.common.context.BaseContextHandler;
import com.github.wxiaoqi.security.common.exception.BaseException;
import com.github.wxiaoqi.security.common.msg.ObjectRestResponse;
import com.github.wxiaoqi.security.common.rest.BaseController;
import com.xxfc.platform.order.biz.OrderActivityInfoDetailBiz;
import com.xxfc.platform.order.contant.enumerate.OrderTypeEnum;
import com.xxfc.platform.order.entity.BaseOrder;
import com.xxfc.platform.order.entity.OrderActivityDetail;
import com.xxfc.platform.order.pojo.dto.ActivityOrderFindDTO;
import com.xxfc.platform.order.pojo.dto.MemberOrderFindDTO;
import com.xxfc.platform.order.pojo.order.ActivityInfoBO;
import com.xxfc.platform.order.service.OrderActivityInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

@RestController
@RequestMapping("tainde/orderActivity")
@Api(value = "活动订单", tags = {"活动订单"})
@Slf4j
public class OrderActivityInfoController extends BaseController<OrderActivityInfoDetailBiz, OrderActivityDetail> {

    @Autowired
    OrderActivityInfoService activityInfoService;

    @Autowired
    UserFeign userFeign;

    @RequestMapping(value = "add", method = RequestMethod.POST)
    @ApiOperation(value = "确认订单")
    @IgnoreClientToken
    public ObjectRestResponse<BaseOrder> add(@RequestBody ActivityInfoBO bo) {
        bo.setAppUserDTO(userFeign.userDetailByToken(BaseContextHandler.getToken()).getData());
        activityInfoService.createOrder(bo);
        return ObjectRestResponse.succ(bo.getOrder());
    }


    @GetMapping(value = "selectList")
    public ObjectRestResponse selectList(ActivityOrderFindDTO activityOrderFindDTO) {
        activityOrderFindDTO.setType(OrderTypeEnum.ACTIVITY.getCode());
        return ObjectRestResponse.succ(baseBiz.selectList(activityOrderFindDTO));
    }

    @GetMapping(value = "selectListStatic")
    public ObjectRestResponse selectListStatic(ActivityOrderFindDTO activityOrderFindDTO) {
        activityOrderFindDTO.setType(OrderTypeEnum.ACTIVITY.getCode());
        return ObjectRestResponse.succ(baseBiz.selectListStatic(activityOrderFindDTO));
    }


    @GetMapping("export")
    public void exportMemberOrders(ActivityOrderFindDTO memberOrderFindDTO, HttpServletResponse response){
        try {
            String name = DateTimeFormatter.ofPattern("YYYYMMddHHmmss").format(LocalDateTime.now());
            response.setContentType("application/vnd.ms-excel;charset=utf-8");
            String filename = String.format("%s-ActivityOrder.xlsx",name);
            response.setHeader("Content-Disposition","attachment;filename="+ new String(filename.getBytes(), "iso8859-1"));
            ServletOutputStream outputStream = response.getOutputStream();
            memberOrderFindDTO.setType(OrderTypeEnum.ACTIVITY.getCode());
            baseBiz.exportMemberOrderData(memberOrderFindDTO, name, outputStream);
            response.setCharacterEncoding("UTF-8");
        }catch (Exception ex){
            log.error(ex.getMessage(),ex);
            throw  new BaseException("导出数据失败");
        }
    }
}